<?php
/*
 * Copyright © Andrea Micheloni 2010
 *
 * Map class
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

require_once("PageTree.class.php");
require_once("Page.class.php");

class Map{
	// Echoes the page map
    public function display($base = '') {
        $tree = $this->getPageTree();

		echo "<ul class=\"map\">\n";
		$tree->displayTreeDFS(Page::getCallBackFunction(), "<li>", "</li>\n", 
			"<ul>", "</ul>\n", $base);
		echo "</ul>\n";
    }

	// Gets the tree of all pages
	private function getPageTree(){	
		$path = GSDATAPAGESPATH;
		$filenames = getFiles($path);
	
		$tree = new PageTree();

		$remaining = Array();

		// Pushes all the pages into $pageList
		foreach ($filenames as $file){
			if(Map::isXml($path . $file)){
				$newPage = Page::loadPage($path . $file);
				array_push($remaining, $newPage);
			}
		}

		// While some pages remain: add them, then decrease $count
		while(count($remaining) > 0){
			$lastRemaining = count($remaining);
			foreach ($remaining as $num=>$page)
				if($tree->addNode($page))
					unset($remaining[$num]);
			if($lastRemaining == count($remaining)) // If this round didn't add nodes
				throw new Exception('Some page parent is non-existant!');
		}

		// Return the created tree
		return $tree;
	}

	// Checks a file to determine if it's xml
	private static function isXml($filename){
    	$ext = pathinfo($filename, PATHINFO_EXTENSION);

		return (strtolower($ext) == 'xml') && is_file($filename);
	}
}
?>
