<?php
/*
 * Copyright © Andrea Micheloni 2010
 *
 * Page class
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

class Page{
	// Attributes of a page
	public $name;
	public $modified;
	public $parent;
	public $private;
	public $title;
	public $description;
	public $keywords;
	public $url;

	// Constructor
	function __construct($xmlData){
		// The XML 'url' node is actually the id. We'll build the url later
		$this->name = 			(string)$xmlData->url;
		$this->modified = 		(string)$xmlData->pubDate;
		$this->parent = 		(string)$xmlData->parent;
		$this->private = 		(string)$xmlData->private;
		$this->title = 			(string)$xmlData->title;
		$this->description =	(string)$xmlData->metad;
		$this->keywords = 		(string)$xmlData->meta;
	}

	// A parent allows us to construct the right url
	public function prependParentUrl($parentUrl){
		if($this->name == 'index')
			$this->url = $parentUrl;
		else
			$this->url = $parentUrl . $this->name . '/';
	}

	// Returns a Page object, given a xml file
	public static function loadPage($xml){
		$data = getXML($xml);
		$page = new Page($data);

		return $page;
	}

	// Return the page callback function
	public static function getCallBackFunction(){
		return array('Page', 'display');	
	}

	// Static callback function to display the page 
	public static function display($page){
		if($page && $page->name){
			$link=find_url($page->name, $page->parent);

			echo "<a href=\"" . $link . "\" class=\"link\" title=\"" . $page->description . "\">" . $page->title . "</a>\n";
			if($page->keywords)
				echo "<span class=\"tags\">[ ".$page->keywords." ]</span>\n";
			if($page->description)
				echo "<span class=\"description\">".$page->description."</span>\n";	
		}
	}
}

?>
