<?php
/*
 * Copyright © Andrea Micheloni 2010
 *
 * PageTree class
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

require_once("Page.class.php");
require_once("Node.class.php");

class PageTree{
	// The constructor
	function __construct(){
		$this->root = new Node();
	}

	// Insert a node in the right place of the tree
	public function addNode($page){
		if(!$page)
			throw new Exception('You are trying to add an empty page.');

		$parentName = $page->parent;
		$parent = NULL;

		// Searching for the parent.
		if($parentName == '')
			$parent = $this->root;
		else
			$parent = $this->findNode($parentName);

		// Have we found it?
		if($parent){
			$child = new Node();

			// Prepend the Url, in case of root or not
			if($parent->page){
				$page->prependParentUrl($parent->page->url);
				if($parent->page->name != $parentName)
					throw new Exception('Problems with addNode.');
			} else
				$page->prependParentUrl('/');

			$child->page = $page;
			$child->children = Array(); 

			return $parent->addChild($page->url, $child);
		} else
			return false;
	}

	// Publicly available procedure to search for a node.
	public function findNode($nodeName){
		return $this->findNodeIn($nodeName, $this->root);
	}

	// Low-level procedure to print the whole tree
	public function displayTreeDFS($callBackFunction, $inElemText = "<li>", 
			$outElemText = "</li>", $inDepthText = "<ul>\n", 
			$outDepthText = "</ul>\n", $node = ''){
		if(!is_callable($callBackFunction))
			throw new Exception('Call back function not callable, sorry.');

		if($node == '')
			$node = $this->root;
		else{
			$node = $this->findNode($node);
			if(!$node)
				throw new Exception('Cannot find node specified for root!');
		}

		return $this->displayTreeDFSIn($callBackFunction, $inElemText, 
			$outElemText, $inDepthText, $outDepthText, $node, false);
	}
	
/* Here be private part */

	// The tree root
	private $root;

	// Search for the node with name $nodeName under $inNode
	private function findNodeIn($nodeName, $inNode){
		if($nodeName == $inNode->getName())
			return $inNode;
		else
			foreach ($inNode->children as $child){
				$found = $this->findNodeIn($nodeName, $child);
				if($found)
					return $found;
			}
	}

	// Procedure to print a tree's portion under $node
	private function displayTreeDFSIn($callBackFunction, $inElemText, $outElemText,
			$inDepthText, $outDepthText, $node, $displayNode = true){
		$displayNode = $displayNode && (strtolower($node->page->private) != 'y');

		if($displayNode){
			echo $inElemText;
			call_user_func($callBackFunction, $node->page);
		}
		$children = $node->children;
		
		if(count($children) > 0){
			if($displayNode)
				echo $inDepthText;

			if(!ksort($children))
				throw new Exception("Can't sort a node's children!");

			foreach ($children as $sortKey => $child){
				$this->displayTreeDFSIn($callBackFunction, $inElemText, 
					$outElemText,$inDepthText, $outDepthText, $child);
			}
			if($displayNode)
				echo $outDepthText;
		}
		if($displayNode)
			echo $outElemText;
	}
}
?>
